@@ -25,6 +25,7 @@
 #include "config.h"
 
 #include <stdint.h>
+#include <stdbool.h>
 
 #ifdef  __cplusplus
 extern "C" {
@@ -284,6 +285,18 @@ bswap32(uint32_t x)
 #endif
 }
 
+/**
+ * Returns whether 64-bit value x is a power of 2.
+ *
+ * @param x a 64-bit value.
+ * @return true if x is a power of 2.
+ */
+static inline bool
+is_pow2_64(uint64_t x)
+{
+	return (x & (x - 1)) == 0;
+}
+
 /**
  * Returns the highest power of two lesser than or equal to 32-bit value x.
  * Saturated to 0 (which isn't a power of two) if x is lesser than 2^0.
