@@ -26,12 +26,11 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <assert.h>
 
 #include <libweston/libweston.h>
-#include "compositor/weston.h"
 #include "weston-test-runner.h"
 #include "weston-test-fixture-compositor.h"
+#include "weston-test-assert.h"
 
 static enum test_result_code
 fixture_setup(struct weston_test_harness *harness)
@@ -54,9 +53,9 @@ PLUGIN_TEST(surface_transform)
 	struct weston_coord_global coord_g;
 
 	surface = weston_surface_create(compositor);
-	assert(surface);
+	test_assert_ptr_not_null(surface);
 	view = weston_view_create(surface);
-	assert(view);
+	test_assert_ptr_not_null(view);
 	surface->width = 200;
 	surface->height = 200;
 	coord_g.c = weston_coord(100, 100);
@@ -66,15 +65,19 @@ PLUGIN_TEST(surface_transform)
 	coord_g = weston_coord_surface_to_global(view, coord_s);
 
 	fprintf(stderr, "20,20 maps to %f, %f\n", coord_g.c.x, coord_g.c.y);
-	assert(coord_g.c.x == 120 && coord_g.c.y == 120);
+	test_assert_f64_eq(coord_g.c.x, 120);
+	test_assert_f64_eq(coord_g.c.y, 120);
 
 	coord_g.c = weston_coord(150, 300);
 	weston_view_set_position(view, coord_g);
 	weston_view_update_transform(view);
 	coord_s = weston_coord_surface(50, 40, surface);
 	coord_g = weston_coord_surface_to_global(view, coord_s);
-	assert(coord_g.c.x == 200 && coord_g.c.y == 340);
+	test_assert_f64_eq(coord_g.c.x, 200);
+	test_assert_f64_eq(coord_g.c.y, 340);
 
 	/* Destroys all views too. */
 	weston_surface_unref(surface);
+
+	return RESULT_OK;
 }
