@@ -31,6 +31,7 @@
 #include "config.h"
 
 #include "weston-test-runner.h"
+#include "weston-test-assert.h"
 #include "vertex-clipping.h"
 
 #define BOX(x1,y1,x2,y2)    { { x1, y1 }, { x2, y2 } }
@@ -59,7 +60,7 @@ assert_vertices(const struct clipper_vertex *clipped, int clipped_n,
 	int first, i, j;
 
 	/* Is the number of clipped vertices correct? */
-	assert(clipped_n == expected_n);
+	test_assert_int_eq(clipped_n, expected_n);
 
 	for (first = 0; first < clipped_n; first++)
 		if (clipper_float_difference(clipped[first].x, expected[0].x) == 0.0f &&
@@ -67,13 +68,13 @@ assert_vertices(const struct clipper_vertex *clipped, int clipped_n,
 			break;
 
 	/* Have we found the first expected vertex? */
-	assert(!clipped_n || first != clipped_n);
+	test_assert_true(!clipped_n || first != clipped_n);
 
 	/* Do the remaining vertices match? */
 	for (i = 1; i < clipped_n; i++) {
 		j = (i + first) % clipped_n;
-		assert(clipper_float_difference(clipped[j].x, expected[i].x) == 0.0f &&
-		       clipper_float_difference(clipped[j].y, expected[i].y) == 0.0f);
+		test_assert_f32_eq(clipper_float_difference(clipped[j].x, expected[i].x), 0.0f);
+		test_assert_f32_eq(clipper_float_difference(clipped[j].y, expected[i].y), 0.0f);
 	}
 }
 
@@ -703,6 +704,8 @@ TEST_P(quad_clip_expected, quad_clip_expected_data)
 	clipped_n = clipper_quad_clip(&quad, tdata->box, clipped);
 
 	assert_vertices(clipped, clipped_n, tdata->clipped, tdata->clipped_n);
+
+	return RESULT_OK;
 }
 
 /* clipper_quad_clip_box32() tests: */
@@ -756,16 +759,22 @@ TEST_P(quad_clip_box32_expected, quad_clip_box32_expected_data)
 	clipped_n = clipper_quad_clip_box32(&quad, &tdata->box32, clipped);
 
 	assert_vertices(clipped, clipped_n, tdata->clipped, tdata->clipped_n);
+
+	return RESULT_OK;
 }
 
 /* clipper_float_difference() tests: */
 
 TEST(float_difference_different)
 {
-	assert(clipper_float_difference(1.0f, 0.0f) == 1.0f);
+	test_assert_f32_eq(clipper_float_difference(1.0f, 0.0f), 1.0f);
+
+	return RESULT_OK;
 }
 
 TEST(float_difference_same)
 {
-	assert(clipper_float_difference(1.0f, 1.0f) == 0.0f);
+	test_assert_f32_eq(clipper_float_difference(1.0f, 1.0f), 0.0f);
+
+	return RESULT_OK;
 }
