@@ -25,6 +25,7 @@
 
 #include "config.h"
 
+#include <assert.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,6 +33,7 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#include <libweston/libweston.h>
 #include "xwayland.h"
 #include "shared/helpers.h"
 
@@ -151,7 +153,7 @@ struct x11_data_source {
 
 static void
 data_source_accept(struct weston_data_source *source,
-		   uint32_t time, const char *mime_type)
+		   uint32_t serial, const char *mime_type)
 {
 }
 
@@ -198,6 +200,9 @@ weston_wm_get_selection_targets(struct weston_wm *wm)
 	char *logstr;
 	size_t logsize;
 
+	if (!seat)
+		return;
+
 	cookie = xcb_get_property(wm->conn,
 				  1, /* delete */
 				  wm->selection_window,
@@ -512,7 +517,6 @@ weston_wm_send_data(struct weston_wm *wm, xcb_atom_t target, const char *mime_ty
 
 	source = seat->selection_data_source;
 	source->send(source, mime_type, p[1]);
-	close(p[1]);
 }
 
 static void
@@ -587,6 +591,7 @@ weston_wm_handle_selection_request(struct weston_wm *wm,
 	weston_log_continue("property %s\n",
 		get_atom_name(wm->conn, selection_request->property));
 
+	assert(selection_request->requestor != wm->selection_window);
 	wm->selection_request = *selection_request;
 	wm->incr = 0;
 	wm->flush_property_on_delete = 0;
@@ -631,6 +636,9 @@ weston_wm_handle_xfixes_selection_notify(struct weston_wm *wm,
 	       xfixes_selection_notify->owner);
 
 	if (xfixes_selection_notify->owner == XCB_WINDOW_NONE) {
+		if (!seat)
+			return 1;
+
 		if (wm->selection_owner != wm->selection_window) {
 			/* A real X client selection went away, not our
 			 * proxy selection.  Clear the wayland selection. */
@@ -714,15 +722,65 @@ weston_wm_set_selection(struct wl_listener *listener, void *data)
 				wm->selection_window,
 				wm->atom.clipboard,
 				XCB_TIME_CURRENT_TIME);
+
+	xcb_flush(wm->conn);
+}
+
+static void
+maybe_reassign_selection_seat(struct weston_wm *wm)
+{
+	struct weston_seat *seat;
+
+	/* If we already have a seat, keep it */
+	if (!wl_list_empty(&wm->selection_listener.link))
+		return;
+
+	seat = weston_wm_pick_seat(wm);
+	if (!seat)
+		return;
+
+	wl_list_remove(&wm->selection_listener.link);
+	wl_list_remove(&wm->seat_destroy_listener.link);
+
+	wl_signal_add(&seat->selection_signal, &wm->selection_listener);
+	wl_signal_add(&seat->destroy_signal, &wm->seat_destroy_listener);
+
+	weston_wm_set_selection(&wm->selection_listener, seat);
+}
+
+static void
+weston_wm_seat_created(struct wl_listener *listener, void *data)
+{
+	struct weston_wm *wm =
+		container_of(listener, struct weston_wm, seat_create_listener);
+
+	maybe_reassign_selection_seat(wm);
+}
+
+static void
+weston_wm_seat_destroyed(struct wl_listener *listener, void *data)
+{
+	struct weston_wm *wm =
+		container_of(listener, struct weston_wm, seat_destroy_listener);
+
+	wl_list_remove(&wm->selection_listener.link);
+	wl_list_init(&wm->selection_listener.link);
+
+	wl_list_remove(&wm->seat_destroy_listener.link);
+	wl_list_init(&wm->seat_destroy_listener.link);
+
+	/* Try to pick another available seat to fall back to */
+	maybe_reassign_selection_seat(wm);
 }
 
 void
 weston_wm_selection_init(struct weston_wm *wm)
 {
-	struct weston_seat *seat;
 	uint32_t values[1], mask;
 
 	wl_list_init(&wm->selection_listener.link);
+	wl_list_init(&wm->seat_create_listener.link);
+	wl_list_init(&wm->seat_destroy_listener.link);
 
 	wm->selection_request.requestor = XCB_NONE;
 
@@ -751,11 +809,20 @@ weston_wm_selection_init(struct weston_wm *wm)
 	xcb_xfixes_select_selection_input(wm->conn, wm->selection_window,
 					  wm->atom.clipboard, mask);
 
-	seat = weston_wm_pick_seat(wm);
-	if (seat == NULL)
-		return;
+	/* Try to set up a selection listener for any existing seat - we
+	 * have a clipboard manager that can copy a subset of available
+	 * selections so they don't disappear when the client owning
+	 * them quits, but to make this work we need to have a seat
+	 * to hang the selection off.
+	 *
+	 * If we have no seat or lose our seat we need to make sure we
+	 * eventually assign a new one, so we listen for seat creation
+	 * and destruction.
+	 */
 	wm->selection_listener.notify = weston_wm_set_selection;
-	wl_signal_add(&seat->selection_signal, &wm->selection_listener);
-
-	weston_wm_set_selection(&wm->selection_listener, seat);
+	wm->seat_destroy_listener.notify = weston_wm_seat_destroyed;
+	wm->seat_create_listener.notify = weston_wm_seat_created;
+	wl_signal_add(&wm->server->compositor->seat_created_signal,
+		      &wm->seat_create_listener);
+	maybe_reassign_selection_seat(wm);
 }
